/* gtkcellrendererpixbuf.c
 * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
	 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
	 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include <stdlib.h>
#include "artycellrendererpixbuf.h"

#define P_(x) x

static void arty_cell_renderer_pixbuf_get_property  (GObject                    *object,
                                                     guint                       param_id,
                                                     GValue                     *value,
                                                     GParamSpec                 *pspec);
static void arty_cell_renderer_pixbuf_set_property  (GObject                    *object,
                                                     guint                       param_id,
                                                     const GValue               *value,
                                                     GParamSpec                 *pspec);
static void arty_cell_renderer_pixbuf_finalize   (GObject                    *object);
static void arty_cell_renderer_pixbuf_get_size   (GtkCellRenderer            *cell,
                                                  GtkWidget                  *widget,
                                                  GdkRectangle               *rectangle,
                                                  gint                       *x_offset,
                                                  gint                       *y_offset,
                                                  gint                       *width,
                                                  gint                       *height);
static void arty_cell_renderer_pixbuf_render     (GtkCellRenderer            *cell,
                                                  GdkDrawable                *window,
                                                  GtkWidget                  *widget,
                                                  GdkRectangle               *background_area,
                                                  GdkRectangle               *cell_area,
                                                  GdkRectangle               *expose_area,
                                                  GtkCellRendererState        flags);


enum {
	PROP_0,
	PROP_FILENAME,
	PROP_DIR,
	PROP_FAIL_PIXBUF,
	PROP_SIZE
};


#define ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), ARTY_TYPE_CELL_RENDERER_PIXBUF, ArtyCellRendererPixbufPrivate))

typedef struct _ArtyCellRendererPixbufPrivate ArtyCellRendererPixbufPrivate;
struct _ArtyCellRendererPixbufPrivate
{
	guint size;
	gchar *dir;
	GdkPixbuf *fail_pixbuf;
};

G_DEFINE_TYPE (ArtyCellRendererPixbuf, arty_cell_renderer_pixbuf, GTK_TYPE_CELL_RENDERER)

static void
arty_cell_renderer_pixbuf_init (ArtyCellRendererPixbuf *cellpixbuf)
{
	ArtyCellRendererPixbufPrivate *priv;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (cellpixbuf);
}

static void
arty_cell_renderer_pixbuf_class_init (ArtyCellRendererPixbufClass *class)
{
	GObjectClass *object_class = G_OBJECT_CLASS (class);
	GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS (class);

	object_class->finalize = arty_cell_renderer_pixbuf_finalize;

	object_class->get_property = arty_cell_renderer_pixbuf_get_property;
	object_class->set_property = arty_cell_renderer_pixbuf_set_property;

	cell_class->get_size = arty_cell_renderer_pixbuf_get_size;
	cell_class->render = arty_cell_renderer_pixbuf_render;

	g_object_class_install_property (object_class,
	                                 PROP_FAIL_PIXBUF,
	                                 g_param_spec_object ("failed-pixbuf",
	                                                      "Failsafe Pixbuf Object",
	                                                      "The pixbuf to render when file loading failed",
	                                                      GDK_TYPE_PIXBUF,
	                                                      G_PARAM_READWRITE));

	g_object_class_install_property (object_class,
	                                 PROP_DIR,
	                                 g_param_spec_string ("dir",
	                                                      "Directory",
	                                                      "The directory where all image files reside.",
	                                                      NULL,
	                                                      G_PARAM_READWRITE));

	g_object_class_install_property (object_class,
	                                 PROP_FILENAME,
	                                 g_param_spec_string ("filename",
	                                                      "Filename",
	                                                      "The filename without extension.",
	                                                      NULL,
	                                                      G_PARAM_READWRITE));

	g_object_class_install_property (object_class,
	                                 PROP_SIZE,
	                                 g_param_spec_uint ("size",
	                                                    "Size",
	                                                    "The size of pixbuf to render",
	                                                    0,
	                                                    200,
	                                                    40,
	                                                    G_PARAM_READWRITE));

	
	g_type_class_add_private (object_class, sizeof (ArtyCellRendererPixbufPrivate));
}

static void
arty_cell_renderer_pixbuf_finalize (GObject *object)
{
	ArtyCellRendererPixbuf *cellpixbuf = ARTY_CELL_RENDERER_PIXBUF (object);
	ArtyCellRendererPixbufPrivate *priv;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (object);

	g_free(cellpixbuf->filename);
	
	g_free(priv->dir);
	if (priv->fail_pixbuf)
		g_object_unref (priv->fail_pixbuf);

	G_OBJECT_CLASS (arty_cell_renderer_pixbuf_parent_class)->finalize (object);
}

static void
arty_cell_renderer_pixbuf_get_property (GObject        *object,
                                        guint           param_id,
                                        GValue         *value,
                                        GParamSpec     *pspec)
{
	ArtyCellRendererPixbuf *cellpixbuf = ARTY_CELL_RENDERER_PIXBUF (object);
	ArtyCellRendererPixbufPrivate *priv;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (object);

	switch (param_id)
	{
		case PROP_FAIL_PIXBUF:
			g_value_set_object (value, priv->fail_pixbuf);
			break;
		case PROP_DIR:
			g_value_set_string (value, priv->dir);
			break;
		case PROP_FILENAME:
			g_value_set_string (value, cellpixbuf->filename);
			break;
		case PROP_SIZE:
			g_value_set_uint (value, priv->size);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
			break;
	}
}

static void
arty_cell_renderer_pixbuf_set_property (GObject      *object,
                                        guint         param_id,
                                        const GValue *value,
                                        GParamSpec   *pspec)
{
	ArtyCellRendererPixbuf *cellpixbuf = ARTY_CELL_RENDERER_PIXBUF (object);
	ArtyCellRendererPixbufPrivate *priv;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (object);

	switch (param_id)
	{
		case PROP_FAIL_PIXBUF:
			if (priv->fail_pixbuf)
				g_object_unref (priv->fail_pixbuf);
			priv->fail_pixbuf = (GdkPixbuf*) g_value_dup_object (value);
			break;
			
		case PROP_DIR:
			if (priv->dir)
				g_free (priv->dir);
			priv->dir = g_value_dup_string (value);
			break;
		case PROP_FILENAME:
			if (cellpixbuf->filename)
				g_free (cellpixbuf->filename);
			cellpixbuf->filename = g_value_dup_string (value);
			break;
		case PROP_SIZE:
			priv->size = g_value_get_uint (value);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
			break;
	}
}

/**
 * arty_cell_renderer_pixbuf_new:
 * 
 * Creates a new #ArtyCellRendererPixbuf. Adjust rendering
 * parameters using object properties. Object properties can be set
 * globally (with g_object_set()). Also, with #GtkTreeViewColumn, you
	 * can bind a property to a value in a #GtkTreeModel. For example, you
 * can bind the "pixbuf" property on the cell renderer to a pixbuf value
 * in the model, thus rendering a different image in each row of the
 * #GtkTreeView.
 * 
 * Return value: the new cell renderer
 **/
GtkCellRenderer *
arty_cell_renderer_pixbuf_new (void)
{
	return g_object_new (ARTY_TYPE_CELL_RENDERER_PIXBUF, NULL);
}

/*static void 
arty_cell_renderer_pixbuf_create_themed_pixbuf (ArtyCellRendererPixbuf *cellpixbuf,
                                                GtkWidget             *widget)
{
	ArtyCellRendererPixbufPrivate *priv;
	GdkScreen *screen;
	GtkIconTheme *icon_theme;
	GtkSettings *settings;
	gint width, height;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (cellpixbuf);

	if (cellpixbuf->pixbuf)
	{
		g_object_unref (cellpixbuf->pixbuf);
		cellpixbuf->pixbuf = NULL;
	}

	screen = gtk_widget_get_screen (GTK_WIDGET (widget));
	icon_theme = gtk_icon_theme_get_for_screen (screen);
	settings = gtk_settings_get_for_screen (screen);

	if (!gtk_icon_size_lookup_for_settings (settings,
	                                        priv->stock_size,
	                                        &width, &height))
	{
		g_warning ("Invalid icon size %u\n", priv->stock_size);
		width = height = 24;
	}

	if (priv->icon_name)
		cellpixbuf->pixbuf = gtk_icon_theme_load_icon (icon_theme,
		                                               priv->icon_name,
		                                               MIN (width, height), 
		                                               GTK_ICON_LOOKUP_USE_BUILTIN,
		                                               NULL);
	else if (priv->gicon)
	{
		GtkIconInfo *info;

		info = gtk_icon_theme_lookup_by_gicon (icon_theme,
		                                       priv->gicon,
		                                       MIN (width, height), 
		                                       GTK_ICON_LOOKUP_USE_BUILTIN);
		if (info)
		{
			cellpixbuf->pixbuf = gtk_icon_info_load_icon (info, NULL);
			gtk_icon_info_free (info);
		}
	}

	g_object_notify (G_OBJECT (cellpixbuf), "pixbuf");
}*/

static GdkPixbuf *
create_colorized_pixbuf (GdkPixbuf *src, 
                         GdkColor  *new_color)
{
	gint i, j;
	gint width, height, has_alpha, src_row_stride, dst_row_stride;
	gint red_value, green_value, blue_value;
	guchar *target_pixels;
	guchar *original_pixels;
	guchar *pixsrc;
	guchar *pixdest;
	GdkPixbuf *dest;

	red_value = new_color->red / 255.0;
	green_value = new_color->green / 255.0;
	blue_value = new_color->blue / 255.0;

	dest = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (src),
	                       gdk_pixbuf_get_has_alpha (src),
	                       gdk_pixbuf_get_bits_per_sample (src),
	                       gdk_pixbuf_get_width (src),
	                       gdk_pixbuf_get_height (src));

	has_alpha = gdk_pixbuf_get_has_alpha (src);
	width = gdk_pixbuf_get_width (src);
	height = gdk_pixbuf_get_height (src);
	src_row_stride = gdk_pixbuf_get_rowstride (src);
	dst_row_stride = gdk_pixbuf_get_rowstride (dest);
	target_pixels = gdk_pixbuf_get_pixels (dest);
	original_pixels = gdk_pixbuf_get_pixels (src);

	for (i = 0; i < height; i++) {
		pixdest = target_pixels + i*dst_row_stride;
		pixsrc = original_pixels + i*src_row_stride;
		for (j = 0; j < width; j++) {		
			*pixdest++ = (*pixsrc++ * red_value) >> 8;
			*pixdest++ = (*pixsrc++ * green_value) >> 8;
			*pixdest++ = (*pixsrc++ * blue_value) >> 8;
			if (has_alpha) {
				*pixdest++ = *pixsrc++;
			}
		}
	}
	return dest;
}


static void
arty_cell_renderer_pixbuf_get_size (GtkCellRenderer *cell,
                                    GtkWidget       *widget,
                                    GdkRectangle    *cell_area,
                                    gint            *x_offset,
                                    gint            *y_offset,
                                    gint            *width,
                                    gint            *height)
{
	ArtyCellRendererPixbuf *cellpixbuf = (ArtyCellRendererPixbuf *) cell;
	ArtyCellRendererPixbufPrivate *priv;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (cell);

	
		if (x_offset) *x_offset = 0;
		if (y_offset) *y_offset = 0;
	
	if (width)
		*width = priv->size;

	if (height)
		*height = priv->size;
/*	if (!cellpixbuf->pixbuf)
	{
		if (priv->stock_id)
			arty_cell_renderer_pixbuf_create_stock_pixbuf (cellpixbuf, widget);
		else if (priv->icon_name || priv->gicon)
			arty_cell_renderer_pixbuf_create_themed_pixbuf (cellpixbuf, widget);
	}

	if (cellpixbuf->pixbuf)
	{
		pixbuf_width  = gdk_pixbuf_get_width (cellpixbuf->pixbuf);
		pixbuf_height = gdk_pixbuf_get_height (cellpixbuf->pixbuf);
	}
	if (cellpixbuf->pixbuf_expander_open)
	{
		pixbuf_width  = MAX (pixbuf_width, gdk_pixbuf_get_width (cellpixbuf->pixbuf_expander_open));
		pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (cellpixbuf->pixbuf_expander_open));
	}
	if (cellpixbuf->pixbuf_expander_closed)
	{
		pixbuf_width  = MAX (pixbuf_width, gdk_pixbuf_get_width (cellpixbuf->pixbuf_expander_closed));
		pixbuf_height = MAX (pixbuf_height, gdk_pixbuf_get_height (cellpixbuf->pixbuf_expander_closed));
	}

	calc_width  = (gint) cell->xpad * 2 + pixbuf_width;
	calc_height = (gint) cell->ypad * 2 + pixbuf_height;

	if (cell_area && pixbuf_width > 0 && pixbuf_height > 0)
	{
		if (x_offset)
		{
			*x_offset = (((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
			              (1.0 - cell->xalign) : cell->xalign) * 
			             (cell_area->width - calc_width));
			*x_offset = MAX (*x_offset, 0);
		}
		if (y_offset)
		{
			*y_offset = (cell->yalign *
			             (cell_area->height - calc_height));
			*y_offset = MAX (*y_offset, 0);
		}
	}
	else
	{
		if (x_offset) *x_offset = 0;
		if (y_offset) *y_offset = 0;
	}

	if (width)
		*width = calc_width;

	if (height)
		*height = calc_height;*/
}

void draw_rectangle_on_pixbuf(GdkPixbuf **pixbuf)
{
	gint pixbuf_width = gdk_pixbuf_get_width(*pixbuf);
	gint pixbuf_height = gdk_pixbuf_get_height(*pixbuf);

	GdkColor color;
	
	color.red = 0;
	color.green = 0;
	color.blue = 0;

	GdkColormap *colormap = gdk_colormap_get_system();
	GdkVisual *visual = gdk_colormap_get_visual (colormap);
	GdkPixmap *pixmap = gdk_pixmap_new (NULL,pixbuf_width,pixbuf_height,visual->depth);
	GdkGC *gc = gdk_gc_new (GDK_DRAWABLE(pixmap));
	gdk_gc_set_colormap (gc,colormap);
	gdk_gc_set_rgb_fg_color (gc,&color);
	gdk_gc_set_line_attributes (gc,1,GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);
	gdk_draw_pixbuf (GDK_DRAWABLE(pixmap),gc,*pixbuf,0,0,0,0,
	                 pixbuf_width,pixbuf_height,
	                 GDK_RGB_DITHER_NONE,0,0);
	gdk_draw_rectangle (GDK_DRAWABLE(pixmap),gc,FALSE,0,0,pixbuf_width-1,pixbuf_height-1);

	g_object_unref (*pixbuf);
	*pixbuf = gdk_pixbuf_get_from_drawable(NULL,GDK_DRAWABLE(pixmap),colormap,
	                                       0,0,0,0,
	                                       pixbuf_width,pixbuf_height);

	g_object_unref (gc);
	g_object_unref (pixmap);
}

static void
arty_cell_renderer_pixbuf_render (GtkCellRenderer      *cell,
                                  GdkWindow            *window,
                                  GtkWidget            *widget,
                                  GdkRectangle         *background_area,
                                  GdkRectangle         *cell_area,
                                  GdkRectangle         *expose_area,
                                  GtkCellRendererState  flags)

{
	//g_print("Rendering .. \n");

	ArtyCellRendererPixbuf *cellpixbuf = (ArtyCellRendererPixbuf *) cell;
	ArtyCellRendererPixbufPrivate *priv;
	GdkPixbuf *pixbuf=NULL;
	//GdkPixbuf *invisible = NULL;
	//GdkPixbuf *colorized = NULL;
	GdkRectangle pix_rect;
	GdkRectangle draw_rect;
	cairo_t *cr;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (cell);


	arty_cell_renderer_pixbuf_get_size (cell, widget, cell_area,
	                                    &pix_rect.x,
	                                    &pix_rect.y,
	                                    &pix_rect.width,
	                                    &pix_rect.height);

	pix_rect.x += cell_area->x + cell->xpad;
	pix_rect.y += cell_area->y + cell->ypad;
	pix_rect.width  -= cell->xpad * 2;
	pix_rect.height -= cell->ypad * 2;

	//g_print("Testing ..");
	if (!gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect) ||
	    !gdk_rectangle_intersect (expose_area, &draw_rect, &draw_rect))
		return;
	//g_print("Testing done ..");

	gchar *escaped = g_uri_escape_string(cellpixbuf->filename,NULL,FALSE);
	gchar *filename = g_strconcat (priv->dir,"/",escaped,".jpg",NULL);
	pixbuf = gdk_pixbuf_new_from_file_at_size (filename,priv->size,priv->size,NULL);
	g_free(escaped);
	g_free(filename);

	if(!pixbuf)
		pixbuf = g_object_ref(priv->fail_pixbuf);

	if(!pixbuf)
		return;

	draw_rectangle_on_pixbuf(&pixbuf);
	
	cr = gdk_cairo_create (window);

	gdk_cairo_set_source_pixbuf (cr, pixbuf, pix_rect.x, pix_rect.y);
	gdk_cairo_rectangle (cr, &draw_rect);
	cairo_fill (cr);
	
	cairo_destroy (cr);

	g_object_unref(pixbuf);
	
	/*ArtyCellRendererPixbuf *cellpixbuf = (ArtyCellRendererPixbuf *) cell;
	ArtyCellRendererPixbufPrivate *priv;
	GdkPixbuf *pixbuf;
	GdkPixbuf *invisible = NULL;
	GdkPixbuf *colorized = NULL;
	GdkRectangle pix_rect;
	GdkRectangle draw_rect;
	cairo_t *cr;

	priv = ARTY_CELL_RENDERER_PIXBUF_GET_PRIVATE (cell);


	arty_cell_renderer_pixbuf_get_size (cell, widget, cell_area,
	                                    &pix_rect.x,
	                                    &pix_rect.y,
	                                    &pix_rect.width,
	                                    &pix_rect.height);

	pix_rect.x += cell_area->x + cell->xpad;
	pix_rect.y += cell_area->y + cell->ypad;
	pix_rect.width  -= cell->xpad * 2;
	pix_rect.height -= cell->ypad * 2;

	if (!gdk_rectangle_intersect (cell_area, &pix_rect, &draw_rect) ||
	    !gdk_rectangle_intersect (expose_area, &draw_rect, &draw_rect))
		return;

	pixbuf = cellpixbuf->pixbuf;
	static int x2=0;
	g_print("Drawing me %d, %d\n",pixbuf,++x2);

	if (cell->is_expander)
	{
		if (cell->is_expanded &&
		    cellpixbuf->pixbuf_expander_open != NULL)
			pixbuf = cellpixbuf->pixbuf_expander_open;
		else if (!cell->is_expanded &&
		         cellpixbuf->pixbuf_expander_closed != NULL)
			pixbuf = cellpixbuf->pixbuf_expander_closed;
	}

	if (!pixbuf)
		return;

	if (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE || !cell->sensitive)
	{
		GtkIconSource *source;

		source = gtk_icon_source_new ();
		gtk_icon_source_set_pixbuf (source, pixbuf);
		 // The size here is arbitrary; since size isn't
		 // wildcarded in the source, it isn't supposed to be
		 // scaled by the engine function
		 
		gtk_icon_source_set_size (source, GTK_ICON_SIZE_SMALL_TOOLBAR);
		gtk_icon_source_set_size_wildcarded (source, FALSE);

		invisible = gtk_style_render_icon (widget->style,
		                                   source,
		                                   gtk_widget_get_direction (widget),
		                                   GTK_STATE_INSENSITIVE,
		                                   // arbitrary 
		                                   (GtkIconSize)-1,
		                                   widget,
		                                   "gtkcellrendererpixbuf");

		gtk_icon_source_free (source);

		pixbuf = invisible;
	}
	else if (priv->follow_state && 
	         (flags & (GTK_CELL_RENDERER_SELECTED|GTK_CELL_RENDERER_PRELIT)) != 0)
	{
		GtkStateType state;

		if ((flags & GTK_CELL_RENDERER_SELECTED) != 0)
		{
			if (gtk_widget_has_focus (widget))
				state = GTK_STATE_SELECTED;
			else
				state = GTK_STATE_ACTIVE;
		}
		else
			state = GTK_STATE_PRELIGHT;

		colorized = create_colorized_pixbuf (pixbuf,
		                                     &widget->style->base[state]);

		pixbuf = colorized;
	}

	cr = gdk_cairo_create (window);

	gdk_cairo_set_source_pixbuf (cr, pixbuf, pix_rect.x, pix_rect.y);
	gdk_cairo_rectangle (cr, &draw_rect);
	cairo_fill (cr);

	cairo_destroy (cr);

	if (invisible)
		g_object_unref (invisible);

	if (colorized)
		g_object_unref (colorized);*/
}

#define __ARTY_CELL_RENDERER_PIXBUF_C__
//#include "gtkaliasdef.c"
